#property copyright "ArteInVolo"
#property link      "https://vk.com/tradelearning"
#property version   "1.00"

#include <Object.mqh>
#include <MovingAverages.mqh>
#include <Arrays\ArrayObj.mqh>
#include "Statistics.mqh"

#property indicator_separate_window
#property indicator_buffers 9
#property indicator_plots   9

#property indicator_label1  "Currency"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrBlue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

#property indicator_label2  "Currency"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrBlack
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

#property indicator_label3  "Currency"
#property indicator_type3   DRAW_LINE
#property indicator_color3  clrRed
#property indicator_style3  STYLE_SOLID
#property indicator_width3  1

#property indicator_label4  "Currency"
#property indicator_type4   DRAW_LINE
#property indicator_color4  clrSilver
#property indicator_style4  STYLE_SOLID
#property indicator_width4  1

#property indicator_label5  "Currency"
#property indicator_type5   DRAW_LINE
#property indicator_color5  clrLime
#property indicator_style5  STYLE_SOLID
#property indicator_width5  1

#property indicator_label6  "Currency"
#property indicator_type6   DRAW_LINE
#property indicator_color6  clrMagenta
#property indicator_style6  STYLE_SOLID
#property indicator_width6  1

#property indicator_label7  "Currency"
#property indicator_type7   DRAW_LINE
#property indicator_color7  clrOrange
#property indicator_style7  STYLE_SOLID
#property indicator_width7  1

#property indicator_label8  "Currency"
#property indicator_type8   DRAW_LINE
#property indicator_color8  clrDeepSkyBlue
#property indicator_style8  STYLE_SOLID
#property indicator_width8  1

#property indicator_label9  "Currency"
#property indicator_type9   DRAW_LINE
#property indicator_color9  clrAqua
#property indicator_style9  STYLE_SOLID
#property indicator_width9  1

// *** Input data *** //

input int InpPeriodFast = 5;
input int InpPeriodSlow = 200;
input int InpNumberOfBars = 500;
input color InpFontColor = Black;
input color InpSignalColor = Lime;
input string InpSymbols = "AUDUSD:EURAUD:EURGBP:EURJPY:EURUSD:GBPUSD:USDCAD:USDCHF:USDJPY";
input string InpActiveCurrency = "EURUSD";

// *** Indicator buffers *** //

double Currency1[], Currency2[], Currency3[], Currency4[], Currency5[], Currency6[], Currency7[], Currency8[], Currency9[];

// *** Global *** //

int FontSize = 13;
int YDistance = 5;
int XDistance = 235;
int ChartWindow = 0;
int MaxBars = 0;
string FontName = "Trebuchet MS";

// *** Classes *** //

class CSymbol : public CObject
{
    public:
        string Name;
        double Close[];
        double SyncClose[];
        double AverageFast[];
        double AverageSlow[];
        datetime Time[];
        
        CSymbol(string name) { Name = name; };
};

// *** Objects *** //

CArrayObj * Currencies;

// *** Functions *** //

void OnDeinit(const int reason) 
{
    delete(Currencies);
    EventKillTimer();
}

int OnInit()
{
    SetIndexBuffer(0, Currency1,  INDICATOR_DATA);
    SetIndexBuffer(1, Currency2,  INDICATOR_DATA);
    SetIndexBuffer(2, Currency3,  INDICATOR_DATA);
    SetIndexBuffer(3, Currency4,  INDICATOR_DATA);
    SetIndexBuffer(4, Currency5,  INDICATOR_DATA);
    SetIndexBuffer(5, Currency6,  INDICATOR_DATA);
    SetIndexBuffer(6, Currency7,  INDICATOR_DATA);
    SetIndexBuffer(7, Currency8,  INDICATOR_DATA);
    SetIndexBuffer(8, Currency9,  INDICATOR_DATA);

    ArraySetAsSeries(Currency1, true);
    ArraySetAsSeries(Currency2, true);
    ArraySetAsSeries(Currency3, true);
    ArraySetAsSeries(Currency4, true);
    ArraySetAsSeries(Currency5, true);
    ArraySetAsSeries(Currency6, true);
    ArraySetAsSeries(Currency7, true);
    ArraySetAsSeries(Currency8, true);
    ArraySetAsSeries(Currency9, true);

    IndicatorSetString(INDICATOR_SHORTNAME, "CurrencyCorrelation");
    IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
    
    MaxBars = TerminalInfoInteger(TERMINAL_MAXBARS);
    ChartWindow = ChartWindowFind();

    GetCurrencyNames();
    
    EventSetTimer(1);

    return(INIT_SUCCEEDED);
}

void OnTimer()
{
    Load();
}

int OnCalculate(
    const int bars, 
    const int counted, 
    const datetime &time[], 
    const double &open[], 
    const double &high[], 
    const double &low[], 
    const double &close[], 
    const long &tickVolume[], 
    const long &volume[], 
    const int &spread[])
{
    int start = 0;
    int copied = Load();

    if (copied > 0)
    {
        Synchronize(start);
        ShowCorrelation(start, bars);
        ShowPanel();
    }

    return(bars);
}

int Load()
{
    for (int i = 0; i < Currencies.Total(); i++)
    {
        CSymbol * s = Currencies.At(i);
        
        int countTimes = CopyTime(s.Name, _Period, 0, MaxBars, s.Time);
        int countCloses = CopyClose(s.Name, _Period, 0, MaxBars, s.Close);

        if (countTimes <= 0 || countCloses <= 0)
        {
            CustomLog(StringFormat("ERROR : No bars for %s", s.Name));
            return(0);
        }
    }

    CustomLog("INFO : History loaded");

    return(1);
}

int Synchronize(const int start)
{
    datetime currentTime = datetime(SeriesInfoInteger(_Symbol, _Period, SERIES_LASTBAR_DATE));

    for (int i = 0; i < Currencies.Total(); i++)
    {
        CSymbol * s = Currencies.At(i);

        ArrayResize(s.SyncClose, MaxBars);
        ArrayResize(s.AverageFast, MaxBars);
        ArrayResize(s.AverageSlow, MaxBars);
        
        ArrayInitialize(s.SyncClose, 0);
        ArrayInitialize(s.AverageFast, 0);
        ArrayInitialize(s.AverageSlow, 0);

        int closeIndex = ArraySize(s.Close) - 1;

        s.SyncClose[MaxBars - 1] = s.Close[closeIndex];

        for (int k = MaxBars - 1; k > start - 1; k--)
        {
            if (GetBarByTime(s.Name, _Period, currentTime) > 0 && closeIndex > 0)
            {
                closeIndex--;
            }

            s.SyncClose[k] = s.Close[closeIndex];            
            currentTime -= PeriodSeconds();
        }

        ExponentialMAOnBuffer(MaxBars, start, 0, InpPeriodFast, s.SyncClose, s.AverageFast);
        ExponentialMAOnBuffer(MaxBars, start, 0, InpPeriodSlow, s.SyncClose, s.AverageSlow);
    }

    return(1);
}

void ShowCorrelation(const int start, const int bars)
{
    CSymbol * s;

    int count = MathMin(bars, MaxBars);

    for (int i = start; i < count; i++)
    {
        int reverseIndex = MathMax(MaxBars - i - 1, 0);
    
        s = Currencies.At(0);
        if (s) Currency1[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(1);
        if (s) Currency2[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(2);
        if (s) Currency3[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(3);
        if (s) Currency4[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(4);
        if (s) Currency5[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(5);
        if (s) Currency6[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(6);
        if (s) Currency7[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(7);
        if (s) Currency8[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
        s = Currencies.At(8);
        if (s) Currency9[i] = s.AverageFast[reverseIndex] / s.AverageSlow[reverseIndex];
    }
}

void ShowPanel()
{
    CSymbol * activeSymbol = Currencies.At(0);

    int positionTop = 0;
    int positionCurrent = 0;
    int symbols = Currencies.Total();

    string activeContent = StringFormat("CURRENCY : %s", InpActiveCurrency);

    ShowLabel("Currency", activeContent, XDistance, YDistance + 10, FontSize - 5, InpFontColor);
    
    for (int i = 0; i < symbols; i++)
    {
        CSymbol * s = Currencies.At(i);
    
        if (StringCompare(s.Name, InpActiveCurrency) == 0)
        {
            activeSymbol = s;
        }
    }

    // Get math values for current symbol

    double activeSequence[];
    double activeSummary = 0;
    
    ArrayResize(activeSequence, ArraySize(activeSymbol.SyncClose));
    
    for (int i = 1; i < MaxBars; i++)
    {
        activeSequence[i] = activeSymbol.SyncClose[i];
        activeSummary += activeSequence[i];
    }

    double activeExpectation = activeSummary / MaxBars;
    double activeVariance = Variance(activeSequence, activeExpectation);
    
    // Compare with other symbols
    
    for (int i = 0; i < symbols; i++)
    {
        CSymbol * s = Currencies.At(i);

        if (StringCompare(s.Name, InpActiveCurrency) == 0)
        {
            continue;
        }

        int startX = XDistance;
        int startY = YDistance + 30;
        int colorIndex = PlotIndexGetInteger(i, PLOT_COLOR_INDEXES);
        color colorValue = color(PlotIndexGetInteger(i, PLOT_LINE_COLOR, 0));
        
        if (i >= symbols / 2)
        {
            startX = 115;
            positionTop = int(symbols / 2);
        }

        double sequence[];
        double summary = 0;
        
        ArrayResize(sequence, ArraySize(s.SyncClose));
        
        for (int k = 1; k < MaxBars; k++)
        {
            sequence[k] = s.SyncClose[k];
            summary += sequence[k];
        }
        
        double expectation = summary / MaxBars;
        double variance = Variance(sequence, expectation);
        double covariance = Cov(activeSequence, sequence, activeExpectation, expectation);
        double correlation = Corr(covariance, activeVariance, variance);
        
        string content = StringFormat("%s : %s", s.Name, DoubleToString(correlation, _Digits));
        
        ShowLabel("Currency" + s.Name, content, startX, startY + 20 * (positionCurrent - positionTop), FontSize - 5, colorValue);
        positionCurrent++;
    }
}

void ShowLabel(const string name, const string value, const int positionX, const int positionY, const int fontSize, const color fontColor)
{
    if (ObjectFind(0, name) < 0)
    {
        ObjectCreate(0, name, OBJ_LABEL, ChartWindow, 0, 0);
    }
    
    ObjectSetString(0, name, OBJPROP_TEXT, value);
    ObjectSetString(0, name, OBJPROP_FONT, FontName);
    ObjectSetInteger(0, name, OBJPROP_COLOR, fontColor);
    ObjectSetInteger(0, name, OBJPROP_FONTSIZE, fontSize);
    ObjectSetInteger(0, name, OBJPROP_ALIGN, ALIGN_RIGHT); 
    ObjectSetInteger(0, name, OBJPROP_XDISTANCE, positionX);
    ObjectSetInteger(0, name, OBJPROP_YDISTANCE, positionY);
    ObjectSetInteger(0, name, OBJPROP_CORNER, CORNER_RIGHT_UPPER); 
}

int GetBarByTime(string symbol, ENUM_TIMEFRAMES period, datetime time)
{
    datetime last;
    SeriesInfoInteger(symbol, period, SERIES_LASTBAR_DATE, last);
    return(Bars(symbol, period, time, last) - 1);
}

int GetCurrencyNames()
{
    string symbols[];
    StringSplit(InpSymbols, ':', symbols);
    
    int symbolCount = ArraySize(symbols);
    Currencies = new CArrayObj();

    for (int i = 0; i < symbolCount; i++)
    {
        CSymbol * s = new CSymbol(symbols[i]);
        
        if (SymbolSelect(s.Name, true))
        {
            Currencies.Add(s);
            PlotIndexSetString(i, PLOT_LABEL, s.Name);
            
            if (s.Name == InpActiveCurrency)
            {
                PlotIndexSetInteger(i, PLOT_LINE_WIDTH, 2);
            }
        }
    }
    
    return(0);
}

void CustomLog(string s)
{
    ShowLabel("Message", s, XDistance, YDistance + 110, FontSize - 5, InpFontColor);
    Print(s);
}